/************************************************************************

 Module:      JTVRemote.cpp

 Project:     JTV

 Description: PV951 remote control driver.

			Version 1.0.4.0 - Win 95/98/2k Test Release


 Prototype (c) 1999 by Mirosaw Grybo   <mgrybos@postinfo.com.pl>
 Copyright (c) 2001 by Nick Zelenkov  <nickola@icfgrp.com>
 Copyright (c) 2001 by Andrew Moroz <dronix@aport.ru>
 Copyright (c) 2001 by Minaichenkov Kirill  'MSG' <minai@mail.ru>

 All rights reserved.

 You may not use any piece of the following code for commercial purposes
 without written permission from the author.

 You may freely copy and distribute the source code as long as it contains
 this copyright notice and the disclaimer. You may freely modify it.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.    IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.

************************************************************************/


#include <unknwn.h>
#include <stdio.h>
#include <windows.h>
#include <winuser.h>

 
#include "jtvrem.h"
#include "jtvremsk.h"

KeyDef keys[]=
{
    { 0, "Key_0", "0", },
    { 1, "Key_1", "1", },
    { 2, "Key_2", "2", },
    { 3, "Key_3", "3", },
    { 4, "Key_4", "4", },
    { 5, "Key_5", "5", },
    { 6, "Key_6", "6", },
    { 7, "Key_7", "7", },
    { 8, "Key_8", "8", },
    { 9, "Key_9", "9", },
	{ 10, "Key_100", "", },

	{ 11, "Key_AirCableTV", "", },
    { 12, "Key_AutoScan", "", },
    { 13, "Key_Teletext", "", },
	{ 14, "Key_TeletextPageUp", "", },
    { 15, "Key_Source", "", },
    { 16, "Key_Mute", "", },
    { 17, "Key_FullScreen", "", },
    { 18, "Key_Power", "Power", },
	{ 19, "Key_MTS", "", },
	{ 20, "Key_Sync", "", },

	{ 22, "Key_TeletextCC", "", },
    { 23, "Key_Recall", "Recall", },
	{ 24, "Key_PlusMinus", "", },
	{ 25, "Key_Function" , "", },
    { 26, "Key_ChannelUp", "ChannelUp", },
    { 27, "Key_VolumeUp", "VolumeUp", },

	{ 29, "Key_TeletextPageDown", "", },
    { 30, "Key_ChannelDown", "ChannelDown", },
    { 31, "Key_VolumeDown", "VolumeDown", }
};

typedef UINT (CALLBACK* LPFNDLLFUNC1)(DWORD,UINT);

typedef int (__cdecl* LPREADI2C)(int,int*);
typedef int (__cdecl* LPWRITEI2C)(int,int,int);
typedef int (__cdecl* LPSETFLAG)(int);
typedef int (__cdecl* LPGETFLAG)(void);


HMODULE hLib;

LPREADI2C  ReadI2C;
LPWRITEI2C WriteI2C;
LPSETFLAG  _SetFlag;
LPGETFLAG  _GetFlag;

int first_delay=1000;
int repeat_delay=100;
BOOL LeaveSoundOnExit = false;
BOOL UseAcorpCompMode = false;
BOOL UseLog = true;
int  System;


int SetFlag (int arg)
{ 
	if (!_SetFlag) 
		return 0;
	else 
		return _SetFlag (arg);
};

int GetFlag (void) 
{
	if (!_GetFlag) 
		return 0;
	else 
		return _GetFlag ();
};

void log(const char* str, ...) {
	
	va_list arglist;

	first_delay      = GetPrivateProfileInt("JTVRemote","KeyFirstDelay",first_delay,".\\jtv.ini");
	repeat_delay     = GetPrivateProfileInt("JTVRemote","KeyRepeatDelay",repeat_delay,".\\jtv.ini");

    LeaveSoundOnExit = ( GetPrivateProfileInt("JTVRemote","LeaveSoundOnExit", 0, ".\\jtv.ini") == 1);
    UseAcorpCompMode = ( GetPrivateProfileInt("JTVRemote","UseAcorpCompMode", 0, ".\\jtv.ini") == 1);
    UseLog =		   ( GetPrivateProfileInt("JTVRemote","UseLog", 1, ".\\jtv.ini") == 1);
   
  
	if (UseLog == 1) 
	{
		FILE* l=fopen("JTVRemote.log","a+t");
		if (l==NULL) return;
		va_start( arglist, str );
		vfprintf(l, str, arglist );
		va_end( arglist );
		fclose(l);
	}
}

int SystemVersion()
{
	OSVERSIONINFOEX osvi;
	BOOL bOsVersionInfoEx;
	int Win_Ver;
	char* Win_Name;

  
	// Try calling GetVersionEx using the OSVERSIONINFOEX structure,
	// which is supported on Windows 2000.
	//
	// If that fails, try using the OSVERSIONINFO structure.

	Win_Ver = -1;
	ZeroMemory(&osvi, sizeof(OSVERSIONINFOEX));
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

	if( !(bOsVersionInfoEx = GetVersionEx ((OSVERSIONINFO *) &osvi)) )
	{
		// If OSVERSIONINFOEX doesn't work, try OSVERSIONINFO.

		osvi.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
		if (! GetVersionEx ( (OSVERSIONINFO *) &osvi) ) 
			return -1;
	}

	switch (osvi.dwPlatformId)
	{
		case VER_PLATFORM_WIN32_NT:

		// Test for the product.

			if ( osvi.dwMajorVersion <= 4 )
			{
				Win_Name = "Microsoft Windows NT ";
				Win_Ver  = 2;
			}
			if ( osvi.dwMajorVersion == 5 )
			{
				Win_Name = "Microsoft Windows 2000 ";
				Win_Ver  = 3;
			}

	         break;

		  case VER_PLATFORM_WIN32_WINDOWS:

			 if ((osvi.dwMajorVersion > 4) || 
	            ((osvi.dwMajorVersion == 4) && (osvi.dwMinorVersion > 0)))
		     {
				 Win_Name = "Microsoft Windows 98 or Millenium";
			} 
			else Win_Name = "Microsoft Windows 95 ";

			Win_Ver  = 1; 
			break;

		case VER_PLATFORM_WIN32s:

			Win_Name = "Microsoft Win32s ";
			Win_Ver  = 0; 
			break;

	}

	// Display version, service pack (if any), and build number.

	 log("> Windows Version - %s version %d.%d %s (Build %d)\n",
		 Win_Name,
         osvi.dwMajorVersion,
         osvi.dwMinorVersion,
         osvi.szCSDVersion,
         osvi.dwBuildNumber & 0xFFFF);

	return Win_Ver; 
}



void AudioInit(void) {

	SetFlag (1);
	WriteI2C( 0x96, 0x2, 0x20 );
	SetFlag (0);
}

void AudioExit(void) {

	SetFlag (1);
	WriteI2C( 0x96, 0x2, 0x10 );
	Sleep(0x14);
	WriteI2C( 0x96, 0x2, 0x10 );
	Sleep(1);
	SetFlag (0);
	SetFlag (1);
	WriteI2C( 0x96, 0x2, 0x40 );
	Sleep(0x14);
	WriteI2C( 0x96, 0x2, 0x40 );
	Sleep(1);
	SetFlag (0);
}


int ReadKey(void) {
	
	int hwsc = 0;
	int res;
	
	SetFlag (1);
	
	if (System == 3) 
	{ // Win2k
		res = ReadI2C (0x97, &hwsc);
		if (hwsc != 0xAA) 
		{ 
			log("> Pressed key code %d\n", hwsc);
			return hwsc; 
		} else {
			return 0xffffffff;
		}
	} else { // Other
		if (UseAcorpCompMode) 
		{ /// acorp  !0 
			res = ReadI2C (0x97, &hwsc);
			SetFlag (0);
		} else { /// MediaForte =0
			ReadI2C (0x97, &hwsc);
			res = SetFlag (0);
		};
		if ((hwsc != 0xAA) && (res == 0)) 
		{ 
			log("> Pressed key code %d\n", hwsc);
			return hwsc; 
		} else {
			return 0xffffffff;
	}
	};
}

BOOL IsJTVWin ()
{
	if (FindWindow("JTVMainWindow", "JTV")==NULL)
	{ return false; } else { return true; }
}

class JTVRemote : public CRemoteDriver {

public:

	JTVRemote(KeyDef *k, int count) : CRemoteDriver(k, count) {
		
		log("> Initializing PV-951 remote module.\n");
		log("> Copyright (c) 2001 by Nick Zelenkov,  <nickola@icfgrp.com>.\n");
        log("> Copyright (c) 2001 by Andrew Moroz,  <dronix@aport.ru>.\n");
		log("> Copyright (c) 2001 by Minaychenkov Kirill 'MSG',  <minai@mail.ru>.\n");
		log("> ------------------------------------------------------------------\n\n");
		log("> KeyFirstDelay=%d, KeyRepeatDelay=%d\n",first_delay,repeat_delay);

		System = SystemVersion();

		switch (System)
		{
			case -1 :   // failed
			case 2 :	// WinNT 4.0

				log("\n> This system don't support.");
				MessageBox(GetDesktopWindow(),"This system don't support.", "Error", MB_OK | MB_ICONERROR);
				exit(1);
				break;

			case 0 :     // Win32s
			case 1 :     // Win32 
			
				log("> Loading pv951utl.dll ...");
				hLib = LoadLibrary("pv951utl.dll");
		
				if (hLib == NULL)
				{  // handle the error
					log("\n> Error during loading ... Can't load PV951Utl.DLL");
					MessageBox(GetDesktopWindow(),"Error during loading ...\n Can't load PV951Utl.DLL", "Error", MB_OK | MB_ICONERROR);
					exit(1);
				}
				
				log("   Ok!\n> Import finctions ...");
				
				ReadI2C  = (LPREADI2C) GetProcAddress(hLib, "DlkI2CSWReadByte");
				WriteI2C = (LPWRITEI2C)GetProcAddress(hLib, "DlkI2CSWWrite3");
				_GetFlag = (LPGETFLAG) GetProcAddress(hLib, "DlkGetEntryFlag");
				_SetFlag = (LPSETFLAG) GetProcAddress(hLib, "DlkSetEntryFlag");

				if (!_SetFlag)
				{  // handle the error
					FreeLibrary(hLib);
					log("> Error during inmort ... Can't import from PV951Utl.DLL");
					MessageBox(GetDesktopWindow(),"Error during inmort ...\nCan't import from PV951Utl.DLL", "Error", MB_OK | MB_ICONERROR);
					exit(1);
				}
				
				log("   Ok!\n");
				break;
			
			case 3 :       // Win2k 
			
				log("> Loading wdmI2cDll.dll ...");
				hLib = LoadLibrary("wdmI2cDll.dll");
		
				if (hLib == NULL)
				{  // handle the error
					log("\n> Error during loading ... Can't load wdmI2cDll.dll");
					MessageBox(GetDesktopWindow(),"Error during loading ...\n Can't load wdmI2cDll.dll", "Error", MB_OK | MB_ICONERROR);
					exit(1);
				}
				
				log("   Ok!\n> Import finctions ...");
				
				ReadI2C  = (LPREADI2C) GetProcAddress(hLib, "readByte");
				WriteI2C = (LPWRITEI2C)GetProcAddress(hLib, "write3byte");
				_GetFlag = (LPGETFLAG) NULL;
				_SetFlag = (LPSETFLAG) NULL;

				if (!WriteI2C)
				{  // handle the error
					FreeLibrary(hLib);
					log("> Error during inmort ... Can't import from wdmI2cDll.dll");
					MessageBox(GetDesktopWindow(),"Error during inmort ...\nCan't import from wdmI2cDll.dll", "Error", MB_OK | MB_ICONERROR);
					exit(1);
				}
				
				log("   Ok!\n");
				break;

		}
	};

	void PollingThread() {
		int last_code=0xffffffff;;
		int value;

		int key_pause = 20;		// new key initial delay
		BOOL isSound = false;	// is sound turned on

		if (IsJTVWin()) 
		{
			log ("> Find JTV Window. Sound is ON.\n");
			Sleep (500);
			AudioInit();
			isSound = true;
		}

		while (WaitForSingleObject(hBreakEvent, 0) != WAIT_OBJECT_0) {
   			Sleep(key_pause);
				
			if (IsJTVWin()) 
			{
				if (!isSound) {
           			log ("> Find JTV Window. Sound is ON.\n");
					Sleep (500);
					AudioInit();
					isSound = true;
				}
			} else {
				if (isSound) {
           			log ("> Lost JTV Window. Sound is OFF.\n");
					Sleep (500);
					AudioExit();
					isSound = false;
				}
			}

			value = ReadKey();

			if (value != 0xffffffff) {
				if (last_code == value) {
					key_pause = repeat_delay;
				} else {
					last_code = value;
					key_pause = first_delay;
				}
					
				// OBSOLETE. Audio Turn on/off on "Mute" pressed
				//if (value == 16) {
				//	if (AudioInit != NULL) {
				//		log("> Try to turn on audio...\n");
				//		(AudioInit)(105,0);
				//	}
				//} else {
				AddStroke(last_code);
				//}
			} else { // no one key is pressed
       			if ((value!=last_code) && (last_code!=0xffffffff)) {
           			last_code = value;
				}
   			};
		};
		
		log("> Break event received. Bye!..\n");
		
		
		if (IsJTVWin())
		{
			if (isSound  && !LeaveSoundOnExit) 
			{
				Sleep (500);
				AudioExit();
			}
		} else { 
			AudioExit();
		}

		if (hLib != NULL) FreeLibrary(hLib);
	};
};

extern "C" {
	__declspec(dllexport) IJTVRemoteControl *__stdcall GetInterfacePtr() {
   	return new JTVRemote(keys, sizeof(keys) / sizeof(keys[0]));
   };
};



